/****************************************************************************
 * Copyright (c) 2019 NVIDIA Corporation.  All rights reserved.
 *
 * Cliff Woolley, 03/06/2019
 *
 * This program replaces the /bin/sh symlink and acts as a conditional
 * redirector so that /bin/sh is either /bin/bash (but without the sh-
 * compatibility mode that you would get if argv[0] was named as "/bin/sh")
 * or /bin/dash.  The selection is chosen based on the (non-)existence of
 * some environment variable.  If that env var is there, then it tells us
 * we've already run our /etc/shinit script (which bash does for us) and we
 * don't need to run it again.  /bin/dash (like any sh-compatible shell)
 * ignores all init scripts when in non-interactive, non-login mode, which
 * leads to bug 200498775.
 ****************************************************************************/

#include <alloca.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static const char *BASH = "/bin/bash";
static const char *DASH = "/bin/dash";

static const char *ENV_KEY = "_CUDA_COMPAT_STATUS";

int main(int argc, const char *const *argv)
{
    const char **argv_new = (const char **)alloca((argc+1)*sizeof(char*));
    memcpy(argv_new, argv, (argc+1)*sizeof(char*));
    argv_new[0] = (getenv(ENV_KEY)) ? DASH : BASH;

#if DEBUG==1
    for (int i=0; i<argc; i++) {
        printf("\"%s\" ", argv_new[i]);
    }
    printf("\n");
#endif

    int rv = execv(argv_new[0], (char *const *)argv_new);
#if DEBUG==1
    printf("ERRNO=%d\n",errno);
#endif
    return rv;
}
